var Backbone = require('backbone')
  , _ = require('underscore')

module.exports = Backbone.View.extend({
  constructor: function(options) {
    this.cid = _.uniqueId('view');
    this._configure(options || {});
    this._ensureElement();
    this.preInitialize.apply(this, arguments);
    this.initialize.apply(this, arguments);
    this.delegateEvents();
  },
  close: function(el) {
    if (this.el != el || this.isRoute === false) {
      return
    }

    this.remove()
    this.unbind()
    this.$el.unbind()
    if (this.onClose) {
      this.onClose()
    }
  },
  context: function() {
    var output = this.data

    if (this.model) {
      output.model = this.model.toJSON()
    }
    if (this.collection) {
      output.collection = this.collection.toJSON()
    }

    return output
  },
  preInitialize: function(options) {
    var that = this
    that.data = {}
    that.rendered = false

    if (that.events) {
      that.events['click a'] = 'transition' 
    } else {
      that.events = { 'click a': 'transition' }
    }

    if (options.parent) {
      that.parent = options.parent
    } else {
      that.parent = 'body'
    }

    Backbone.trigger('closeView', that.el)
    that.listenTo(Backbone, 'closeView', that.close)
  },
  render: function() {
    var ns = 'template'
    var template = Handlebars.template(require(ns + '/' + this.template))
    var content = template(this.context())

    $(this.parent).append(this.$el.html(content))
  },
  transition: function(e) {
    var route = Backbone.$(e.currentTarget).attr('href')

    if (route[0] == '/') {
      route = route.slice(1, route.length)
    }
    Backbone.history.navigate(route, { trigger: true }) 

    return false
  }
})
