var async = require('async')
  , fs = require('fs')
  , path = require('path')

var Backbone = require('backbone')
  , Handlebars = require('handlebars')
  , _ = require('underscore')

Backbone.$ = function() {
  return {
    attr: function() {}
  }
}

module.exports = Backbone.View.extend({
  constructor: function(options) {
    this.cid = _.uniqueId('view');
    this._configure(options || {});
    this._ensureElement();
    this.preInitialize.apply(this, arguments);
    this.initialize.apply(this, arguments);
    this.delegateEvents();
  },
  context: function() {
    var output = this.data
    if (this.model) {
      output.model = this.model.toJSON()
    }
    if (this.collection) {
      output.collection = this.collection.toJSON()
    }

    return output
  },
  listenTo: function() {
  },
  preInitialize: function(options) {
    var that = this
    that.data = {}
    that.layoutData = {}
    that.events = null
    that.config = {
      layoutPath:  'app/template/layout',
      templatePath:  'app/template',
      layout: 'app',
      layoutViews: []
    }
    that.renderLayout = true


    if (typeof(options.renderLayout) !== 'undefined') {
      that.renderLayout = options.renderLayout
    }

    if (Backbone.View.config) {
      if (Backbone.View.config.layoutPath) {
        that.config.layoutPath = Backbone.View.config.layoutPath 
      }

      if (Backbone.View.config.templatePath) {
        that.config.templatePath = Backbone.View.config.templatePath 
      }

      if (Backbone.View.config.layout) {
        that.config.layout = Backbone.View.config.layout 
      }
      
      if (Backbone.View.config.layoutViews) {
        that.config.layoutViews = Backbone.View.config.layoutViews
      }
    }
  },
  render: function(callback) {
    var that = this;
    async.auto({
      template: function(subCallback) {
        fs.readFile(
          path.join(
            that.config.templatePath, 
            that.template + '.handlebars'
          ),
          'ascii',
          subCallback
        )
      },
      layout: function(subCallback) {
        if (that.config.layout != null && that.renderLayout) {
          fs.readFile(
            path.join(
              that.config.layoutPath,
              that.config.layout + '.handlebars'
            ),
            'ascii',
            subCallback
          )
        } else {
          subCallback(null, null)
        }
      },
      layoutViews: function(subCallback) {
        if (!that.renderLayout) {
          return subCallback(null, null)
        }

        async.map(that.config.layoutViews, function(viewFn, mapCallback) {
          new viewFn({ callback: mapCallback, renderLayout: false })
        },
        function(err, results) {
          if (err) {
            throw new Error(err.message)
            return subCallback(err)
          } 

          var data = {}
          for (var i = 0; i < that.config.layoutViews.length; i++) {
            data[that.config.layoutViews[i].prototype.template] = results[i]
          }

          return subCallback(null, data)
        })
      }
    },
    function(err, results) {
      if (err) {
        throw new Error(err.message)
        return callback(err)
      }

      var template = Handlebars.compile(results.template)
      if (results.layout === null) {
        return callback(null, template(that.context()))
      } else {
        var layoutData = that.layoutData
        layoutData.body = template(that.context())
        for (var part in results.layoutViews) {
          var name = part.replace('/', '_')
          layoutData[name] = results.layoutViews[part]
        }

        layout = Handlebars.compile(results.layout)
        return callback(null, layout(layoutData))
      }
    })
  },
})
